/**
 * @file main.c
 * @author Jake G
 * @date 15 June 2024
 * @brief File contains the main function.
 *
 * This file contains the main logic loop and exists to setup the values 
 * specific to the micro-controller. All other functions and configuration are
 * extracted into separate source files and headers for configuration.
 */

#define F_CPU 3333333UL

//This can prevent issues with utils/delay.h library with the gcc toolchain
#define __DELAY_BACKWARD_COMPATIBLE__

#include "config.h"
#include "RegEdit.h"
#include "zero_cross_detection.h"
#include "ADC.h"
#include "TriacOut.h"
#include "load.h"
#include <avr/cpufunc.h> /* Required header file */
#include <avr/io.h>
#include <util/delay.h>



//Set the function pointer for the delay func
void (*Delay_MicroSeconds)(double us) = _delay_us;

static void setEnablePinsHigh(void)
{
    //Pins 12, 6 and 7 are all set high.
    PORTA.DIR |= (1<<2); //PA2= pin 12
    PORTB.DIR |= (1<<2)|(1<<3); //PB2 = pin 7, PB3 = pin 6
    
    PORTA.OUT |= (1<<2);
    PORTB.OUT |= (1<<2)|(1<<3);
}

int main(int argc, char **argv)
{
    while(true){
        for(int i = 0; i < GatePulsesQty; i++){
            ZCD_Poll();
            _delay_us(Tau);
            TriacOut_SetupPins();
            TriacOut_SetAllHigh();  //Only G1 exists in High power mode
            TriacOut_PulsePins(GatePulses[i]);
        }
        
        //The G1 pin is low at this point.
        _delay_ms(2500);
        ZCD_Poll();
        //setEnablePinsHigh();
        while(true){
            ;   //Do nothing until new Power cycle/reset occurs
        }
    }
}
